// Copyright 2024 The Dawn & Tint Authors
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its
//    contributors may be used to endorse or promote products derived from
//    this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "src/tint/lang/hlsl/type/byte_address_buffer.h"

#include <gtest/gtest.h>

#include "src/tint/lang/core/type/f32.h"
#include "src/tint/lang/core/type/i32.h"

namespace tint::hlsl::type {
namespace {

TEST(HlslTypeByteAddressBuffer, Equals) {
    core::type::F32 f{};
    core::type::I32 i{};

    const ByteAddressBuffer a(&f, core::Access::kRead);
    const ByteAddressBuffer b(&f, core::Access::kRead);
    const ByteAddressBuffer c(&f, core::Access::kReadWrite);
    const ByteAddressBuffer d(&i, core::Access::kRead);

    EXPECT_TRUE(a.Equals(b));
    EXPECT_FALSE(a.Equals(c));
    EXPECT_FALSE(a.Equals(d));
    EXPECT_FALSE(a.Equals(i));
}

TEST(HlslTypeByteAddressBuffer, FriendlyName) {
    core::type::F32 f{};
    const ByteAddressBuffer l(&f, core::Access::kReadWrite);

    EXPECT_EQ(l.FriendlyName(), "hlsl.byte_address_buffer<f32, read_write>");
}

}  // namespace
}  // namespace tint::hlsl::type
